@@ -245,7 +245,8 @@ def submit_valset_packet(key_value_dict, serial_stream, timeout=TIMEOUT_DEF):
     sorted_dict = dict(sorted(key_value_dict.items(), key=lambda x : x[0].value))
     for i,(key_id,value) in enumerate(sorted_dict.items()):
         
-        n_transaction = 2 if i else 1
+        #n_transaction = 2 if i else 1
+        n_transaction = 1
         payload = struct.pack('<BBBB', 1, 7, n_transaction, 0)
 
         logger.debug('submit_valset_packet::key_id [ {} ] --> [ {} ] (transaction {})'.format(key_id, value, n_transaction))
@@ -284,6 +285,8 @@ def wait_for_ack_nak(packet_type, serial_stream, timeout=TIMEOUT_DEF):
             logger.debug('packet [ {} ] unsupported'.format(PacketType(packet[0:2])))
             continue
 
+        logger.debug(f'Received packet {incoming_packet_type}')
+
         if incoming_packet_type == PacketType.ACK_ACK:
             if __ack_nck_corresponds_to_packet__(packet_type, parsed_packet):
                 return True
@@ -378,6 +381,7 @@ def submit_valset_from_valget_line(line, serial_stream):
     if not line.startswith('CFG-VALGET'):
         return False
     
+    logger.debug(f'Sending contents of line [ {line} ]')
     packet = line.split(' - ')[1]
     payload_str = packet[SKIP_NBYTES*3:]
     
@@ -586,6 +590,24 @@ KEY_ID_VALUE_FORMATS = {
     KeyId.CFG_UART1_BAUDRATE : 'L',
     KeyId.CFG_UART2_BAUDRATE : 'L',
     KeyId.CFG_NAVSPG_USE_PPP : '?',
+    KeyId.CFG_MSGOUT_UBX_RXM_RAWX_USB : '?',
+    KeyId.CFG_MSGOUT_UBX_NAV_POSECEF_USB : '?',
+    KeyId.CFG_MSGOUT_UBX_RXM_SFRBX_USB : '?',
+    KeyId.CFG_SIGNAL_GPS_L1CA_ENA : '?',
+    KeyId.CFG_SIGNAL_GPS_L2C_ENA : '?',
+    KeyId.CFG_SIGNAL_GAL_E1_ENA : '?',
+    KeyId.CFG_SIGNAL_GAL_E5B_ENA : '?',
+    KeyId.CFG_SIGNAL_BDS_B1_ENA : '?',
+    KeyId.CFG_SIGNAL_BDS_B2_ENA : '?',
+    KeyId.CFG_SIGNAL_QZSS_L1CA_ENA : '?',
+    KeyId.CFG_SIGNAL_QZSS_L2C_ENA : '?',
+    KeyId.CFG_SIGNAL_GLO_L1_ENA : '?',
+    KeyId.CFG_SIGNAL_GLO_L2_ENA : '?',
+    KeyId.CFG_SIGNAL_GPS_ENA : '?',
+    KeyId.CFG_SIGNAL_GAL_ENA : '?',
+    KeyId.CFG_SIGNAL_BDS_ENA : '?',
+    KeyId.CFG_SIGNAL_QZSS_ENA : '?',
+    KeyId.CFG_SIGNAL_GLO_ENA : '?'
 }
 
 # ------------------------------------------------------------------------------
