@@ -14,24 +14,43 @@ from roktools.time import weektow_to_datetime
 from . import helpers
 from . import config as ublox_config
 from . import core
-from . import SERIAL_PORT_STR
+from . import SERIAL_PORT_STR, BAUD_RATE_STR
 
-def autodetect():
+def autodetect_serial_port():
     """
     Attempt to find a serial port in which the receiver is connected. 
 
     :return: If found, a string with the device specification. Otherwise, None
     """
 
+    serial_port = None
+
     for port in serial.tools.list_ports.comports():
         if 'u-blox GNSS receiver' in port:
-            res = {
-                'receiver': 'u-blox',
-                SERIAL_PORT_STR: port.device
-            }
-            return res
+            serial_port = port.device
+            break
 
-    return None
+    return serial_port
+
+# ------------------------------------------------------------------------------
+
+def autodetect_baud_rate(serial_port):
+   
+    BAUD_RATE_LIST = [9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600]
+
+    baudrate = None
+    for baud_rate_candidate in BAUD_RATE_LIST:
+        with serial.Serial(serial_port, baud_rate_candidate) as ser:
+            if ser.isOpen(): ser.close()
+            ser.open()
+            doc = ser.read(1000)
+            ser.close()
+
+            if b'\xb5\x62' in doc:                        
+                baudrate = baud_rate_candidate
+                break
+        
+    return baudrate
 
 # ------------------------------------------------------------------------------
 
@@ -51,18 +70,20 @@ def reset(serial_port):
 
 # ------------------------------------------------------------------------------
 
-def submit_config(doc, serial_port=None):
+def submit_config(doc, serial_port=None, baudrate=None):
     """
     Submit a configuration (using the Receiver configuration standard in JSON
     format)
     """
 
     if not serial_port:
-        res = autodetect()
-        serial_port = res[SERIAL_PORT_STR]
+        serial_port = autodetect_serial_port()
+
+    if not baudrate:
+        baudrate = autodetect_baud_rate(serial_port)
 
     try:
-        serial_stream = serial.Serial(serial_port)
+        serial_stream = serial.Serial(serial_port, baudrate)
         ublox_config.set_from_dict(serial_stream, doc)
         serial_stream.close()
         
@@ -74,67 +95,72 @@ def submit_config(doc, serial_port=None):
 
 # ------------------------------------------------------------------------------
 
-def detect_config(serial_port=None, messages=False):
+def detect_config(serial_port=None, baudrate=None, messages=False):
 
     config = {}
 
     try:
-        config = detect_connection(serial_port=serial_port)
+        config = detect_connection(serial_port=serial_port, baudrate=baudrate)
+        serial_port = config.pop(SERIAL_PORT_STR)
+        baudrate = config.pop(BAUD_RATE_STR)
     except Exception as e:
-        logger.critical('Could not detect connection: {}'.format(str(e)))
+        logger.critical('Could not detect connection: {}. Re-run command'.format(str(e)))
         return None
 
-    serial_port = config.pop(SERIAL_PORT_STR)
-
-    serial_stream = serial.Serial(serial_port, **config)
-
-    try:
-        config = ublox_config.get(serial_stream)
-        config[SERIAL_PORT_STR] = serial_port
-    except Exception as e:
-        logger.critical(f'Unable to get receiver parameters. Re-launch the command::{str(e)}')
-        return None
-
-    if messages:
-
-        packet_types = set()
+    with serial.Serial(serial_port, baudrate, **config) as serial_stream:
 
         try:
-            t_start = time.time()
+            config = ublox_config.get(serial_stream)
+            config[SERIAL_PORT_STR] = serial_port
+            config[BAUD_RATE_STR] = baudrate
+        except Exception as e:
+            logger.critical(f'Unable to get receiver parameters. Re-launch the command::{str(e)}')
+            return None
 
-            for packet in core.ublox_packets(serial_stream, core.TIMEOUT_DEF):
+        if messages:
 
-                elapsed_time = time.time() - t_start
+            packet_types = set()
 
-                ptype = packet[0:2]
-                if ptype not in packet_types:
-                    packet_types.add(ptype)
+            try:
+                t_start = time.time()
 
-                if (elapsed_time > 10):
-                    break
-        except TimeoutError:
-            pass
+                for packet in core.ublox_packets(serial_stream, core.TIMEOUT_DEF):
 
-        config['messages'] = [core.PacketType(m).name for m in packet_types]
+                    elapsed_time = time.time() - t_start
 
-    logger.debug('Closing serial stream')
-    serial_stream.close()
+                    ptype = packet[0:2]
+                    if ptype not in packet_types:
+                        packet_types.add(ptype)
+
+                    if (elapsed_time > 10):
+                        break
+            except TimeoutError:
+                pass
+
+            config['messages'] = [core.PacketType(m).name for m in packet_types]
 
     return config
 
 # ------------------------------------------------------------------------------
 
 
-def detect_connection(serial_port=None, baudrate=38400):
+def detect_connection(serial_port=None, baudrate=None):
 
     if not serial_port:
         logger.debug('No serial port specified, attempting to find one')
-        serial_port = autodetect()[SERIAL_PORT_STR]
+        serial_port = autodetect_serial_port()
+
+    if not baudrate:
+        logger.debug('No baud rate specified, attempting to find one')
+        baudrate = autodetect_baud_rate(serial_port)
 
     stream = serial.Serial(serial_port, baudrate=baudrate, timeout=core.TIMEOUT_DEF)
-    logger.debug(f'Opened [ {serial_port} ] --> [ {stream} ]')
+    logger.debug(f'Opened connection [ {serial_port} ] --> [ {stream} ]')
 
-    config = {SERIAL_PORT_STR: serial_port}
+    config = {
+        SERIAL_PORT_STR: serial_port,
+        BAUD_RATE_STR: baudrate
+    }
 
     KEY_IDS = [core.KeyId.CFG_UART1_BAUDRATE, core.KeyId.CFG_UART1_STOPBITS, 
                 core.KeyId.CFG_UART1_PARITY, core.KeyId.CFG_UART1_DATABITS]
@@ -144,7 +170,7 @@ def detect_connection(serial_port=None, baudrate=38400):
     key_id = core.KeyId.CFG_UART1_BAUDRATE
     if key_id in key_values:
         value = key_values[key_id]
-        config['baudrate'] = core.parse_key_id_value(key_id, value)
+        config[BAUD_RATE_STR] = core.parse_key_id_value(key_id, value)
 
     key_id = core.KeyId.CFG_UART1_STOPBITS
     if key_id in key_values:
@@ -190,7 +216,7 @@ def detect_connection(serial_port=None, baudrate=38400):
 class Recorder(threading.Thread):
 
 
-    def __init__(self, serial_port=None, file_rotation=rinex.FilePeriod.DAILY,
+    def __init__(self, serial_port=None, baudrate=None, file_rotation=rinex.FilePeriod.DAILY,
                  output_dir=".", receiver_name="UBLX"):
         threading.Thread.__init__(self)
 
@@ -205,8 +231,14 @@ class Recorder(threading.Thread):
             logger.info('Serial port not defined, autodetecting')
             connection_config = detect_connection()
             self.serial_port = connection_config.pop(SERIAL_PORT_STR)
+            self.baudrate = connection_config.pop(BAUD_RATE_STR)
 
-        self.serial_stream = serial.Serial(self.serial_port)
+        if baudrate:
+            self.baudrate = baudrate
+        if not self.baudrate:
+            self.baudrate = autodetect_baud_rate(self.serial_port)
+
+        self.serial_stream = serial.Serial(self.serial_port, self.baudrate)
 
         if not self.serial_stream.is_open:
             logger.critical(f'Could not open serial stream [ {self.serial_port} ]')
